/*********************************************************************
;* FileName     : SRAM_driver.c
;* Dependencies : See INCLUDES section below
;* Processor    : PIC16F family of PICmicro - 8 bit MCU      
;* Assembler/
;* Linker       : HI-TECH PICC STD 9.60/ PRO 9.65
;* Company      : Microchip Technology, Inc.
;*
;*                     Software License Agreement                      
;*                                                                     
;* Microchip Technology Inc. (Microchip) licenses this software to you
;* solely for use with Microchip microcontrollers and Microchip serial
;* EEPROM products.  The software is owned by Microchip and/or its
;* licensors, and is protected under applicable copyright laws.  All
;* rights reserved.
;*
;* SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
;* DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
;* INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
;* NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
;* LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
;* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
;* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
;* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
;* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
;*
;* Author                   Date                Version         Comment
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;* Deepak Kumar Rana        20-Feb-08           V1.0            Original
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/


#include "SRAM_driver.h";

/**************************************************************************
; Function              :Init_SPI
; Precondition          :Make sure that the controller has an on chip MSSP module 
; Side Effect           :Make sure that the SPI mode and the SCK frequecy are as per the specifications of SRAM  
; Overview              :Initializes the MSSP moudle of the controller for SPI mode 1
;                        CKP =0 CKE=1
**************************************************************************/



void Init_SPI()
{
	TRISC2 =0;						// CS pin is  output
	TRISC3 =0;						// CLK pin is output
	TRISC4 =1;						// SDI pin is input
    TRISC5 =0;						// SDO pin is output
    
    CE	=1;							// SRAM is disabled
    SSPCON2 =0;						// clear all the flags
    SSPSTAT =0;						// Input data sampled at the middle of data output time SMP=0
    CKE=1;							//Select the SPI mode 1
    SSPCON =0;						//CKP=0,SCK=FOSC/4							
    SSPEN=1	;						//enable the SCK,SDO,SDI pins
	
   
}


/**************************************************************************
; Function              :Init_Datamem
; Precondition          :The data to be written to Data_Writepage is passed in the fuction call from SRAM_main.c
; Side Effect           :We are writing the same data to all the 32 bytes to the location for demo. In your
;						 application modify the routine appropriately
; Overview              :Before writing data in Page or Sequence mode we need store the data in the data memory location of the 
						 controller.
**************************************************************************/


void Init_Datamem(unsigned char Data_Out, unsigned char count)
{

         unsigned char temp;
         for (temp =0;temp<count;temp++)    // Initialize the specified data memory locations 
         Data_Writepage[temp]=Data_Out;		// With data value passed from the SRAM_main.c
}
			



/**************************************************************************
; Function              : SRAMWritePage  
; Precondition          : Configure SRAM for page mode before calling this routine.The address location and number of 
;						  bytes to be written are passed from the fuction call in SRAM_main.c 
; Side Effects          : User needs to make sure that he is writing within the page  
; Overview              : The data stored in the controller data memory location Data_Writepage is written to the SRAM
;                        
**************************************************************************/

void SRAMWritePage(unsigned char Add_High,unsigned char Add_Low,unsigned char count)
{ 	
	  unsigned char temp,i;
	  CE =0 ;		  		 				//Enable SRAM
	  
	  SSPBUF =Write;   						//Write to commad to SR
	  while(!BF);							// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
	  
	  SSPBUF =Add_High;   					//High Address byte of the SRAM
	  while(!BF);							// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag

	  SSPBUF = Add_Low;   					//Low Address byte of the SRAM
	  while(!BF);							//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
     
      for(i=0;i<count;i++)					//Writing to page of the SRAM
     {
      SSPBUF = Data_Writepage[i];   		//Write the data to the SRAM address location
	  while(!BF);							// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
	 }
	 
      CE =1;								// Disable the SRAM 


}


/**************************************************************************
; Function              : SRAMReadPage  
; Precondition          : SRAM needs to be configured for page mode before calling this routine.The address location and number of 
;						  bytes to be read are passed from the fuction call in SRAM_main.c
; Side Effects          : User needs to make sure that he is reading within the page    
; Overview              : The Number of bytes requested are read from the specified SRAM address location onwards                         
**************************************************************************/
  
 


void SRAMReadPage(unsigned char Add_High,unsigned char Add_Low,unsigned char count)
{ 	
	  unsigned char temp,i;
	  CE =0 ;		  		 				//Enable SRAM
	  
	  SSPBUF =Read;   						//Read commad to SR
	  while(!BF);							//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
	  
	  SSPBUF =Add_High;   					//High Address byte of the SRAM
	  while(!BF);							//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag

	  SSPBUF = Add_Low;   					//Low Address byte of the SRAM
	  while(!BF);							//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
     
      for(i=0;i<count;i++)
     {
      SSPBUF = 0xFF;   		    		   //Dummy write to the SSPBUF
	  while(!BF);			   			   //Check for the BF flag to verify data reception
	  Data_Readpage[i] = SSPBUF;		   //Store the data read in the data memory of the controller
	 }
	 
      CE =1;							  // Disable the SRAM 


}



/**************************************************************************
; Function              :SRAMWriteSeq
; PreCondition          :User needs to make sure that the SRAM has be configured to Sequential mode.The address location and number of 
;						 bytes to be written are passed from the fuction call in SRAM_main.c  
; Side Effects          :None
; Overview              :The data stored in the controller data memory location Data_Writepage is written to the SRAM                  
						 In Sequence Mode we can write across the page boundary.	
**************************************************************************/
  
 

void SRAMWriteSeq(unsigned char Add_High,unsigned char Add_Low,unsigned char count) 
{
    unsigned char temp,i;
	 CE =0 ;		  		 				//Enable SRAM
	  
	  SSPBUF =Write;   						//Write to commad to SR
	  while(!BF);							// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
	  
	  SSPBUF =Add_High;   					//High Address byte of the SRAM
	  while(!BF);							// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag

	  SSPBUF = Add_Low;   					//Low Address byte of the SRAM
	  while(!BF);							//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
     
      for(i=0;i<count;i++)
     {
      SSPBUF = Data_Writepage[i];   		//Write to the SRAM
	  while(!BF);					        //Check for the BF flag to ensure trasmission
	  temp = SSPBUF;						//Clear the BF flag
	 }
	 
      CE =1;								// Disable the SRAM 
}





/**************************************************************************
; Function              :SRAMReadSeq
; Precondition          :User needs to make sure that the SRAM has been configured to Sequential mode.The address location and number of 
;						 bytes to be read are passed from the fuction call in SRAM_main.c 
; Side Effects          :None 
; Overview              :The Number of bytes requested are read from the specified SRAM address location onwards into the Data_Readpage
						 memory location.                 
**************************************************************************/
  
 
void SRAMReadSeq(unsigned char Add_High,unsigned char Add_Low,unsigned char count)
{

	 unsigned char temp,i;
	 CE =0 ;		  					//Enable SRAM
	  
	  SSPBUF =Read;   					//Read commad to SR
	  while(!BF);						// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;					//Clear the BF flag
	  
	  SSPBUF =Add_High;   				//High Address byte of the SRAM
	  while(!BF);						// Check for the BF flag to ensure trasmission
	  temp = SSPBUF;					//Clear the BF flag

	  SSPBUF = Add_Low;   				//Low Address byte of the SRAM
	  while(!BF);						//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;					//Clear the BF flag
     
      for(i=0;i<count;i++)
     {
      SSPBUF = 0xFF;   		   			 //Dummy write operation to SSPBUF read from SRAM
	  while(!BF);			    		 //Check for the BF flag to verify data reception
	  Data_Readpage[i] = SSPBUF;	     //Store the data in the data memory of the controller
	 }
	 
      CE =1;							// Disable the SRAM 


}


/**************************************************************************
; Function              :SRAMWriteByte
; Precondition          :Make sure that before calling this fuction the status register of SRAM is configured for Byte Mode 
; Side Effects          :None
; Overview              :This performs the operation of writing byte stored in Data_Out location to SRAM. 
;						                      
**************************************************************************/
  

void SRAMWriteByte( unsigned char Add_High,unsigned char Add_Low,unsigned char Data_Out)
{
      unsigned char temp;
	  CE =0 ;		  		 		//Enable SRAM
	  
	  SSPBUF =Write;   				//Write to commad to SR
	  while(!BF);					//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			    //Clear the BF flag
	  
	  SSPBUF =Add_High;   		    //High Address byte of the SRAM
	  while(!BF);			        //Check for the BF flag to ensure trasmission
	  temp = SSPBUF;				//Clear the BF flag

	  SSPBUF = Add_Low;   		   //Low Address byte of the SRAM
	  while(!BF);				   //Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			   //Clear the BF flag

      SSPBUF = Data_Out;   		   //Write to commad to SR
	  while(!BF);				   //Check for the BF flag to ensure trasmission
	  temp = SSPBUF;		       //Clear the BF flag
	  
	  CE =1;					   //Disable the SRAM 


}



/**************************************************************************
; Function              :SRAMReadByte
; Precondition          :Make sure that before calling this fuction the status register of SRAM is configured for Byte Mode 
; Side Effects          :After the end of this function Data_In contains the data read from the SRAM
; Overview              :This performs the operation of reading byte from SRAM.The read content is stored at location Data_In 
;						                      
**************************************************************************/

unsigned char SRAMReadByte( unsigned char Add_High,unsigned char Add_Low)
{
      unsigned char temp;
	  CE =0 ;		  				 //Enable SRAM
	  
	  SSPBUF =Read;   				//Read commad to SR
	  while(!BF);					//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;				//Clear the BF flag
	  
	  SSPBUF =Add_High;   			//High Address byte of the SRAM
	  while(!BF);					//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;				//Clear the BF flag

	  SSPBUF = Add_Low;   			//High Address byte of the SRAM
	  while(!BF);					//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;				//Clear the BF flag

      SSPBUF = 0xFF;   				//Dummy write to SSPBUF to read from SRAM
	  while(!BF);					//Check for the BF flag to verify data reception
	  temp = SSPBUF;				//Copy the content to controller data memory
      
      CE =1;				        // Disable the SRAM 
   
      return (temp);    		   // pass the read data back	
}


void SRAMWriteStatusReg(unsigned char i)
  
  {
	  unsigned char temp;
	  CE =0 ;		  		 	  //Enable SRAM
	  
	  SSPBUF = WRSR;   			 //Write to commad to SR
	  while(!BF);				 // Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			 //Clear the BF flag
	  
	  SSPBUF = i;   			 //Write the mode passed from the main fuctionto SR
	  while(!BF);			 	 //Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			 //Clear the BF flag
	  
	  CE =1;					 // Disable the SRAM 

  }


unsigned char SRAMReadStatusReg()
  
  {
	  unsigned char temp;
	  CE =0 ;		  		 	//Enable SRAM
	  
	  SSPBUF = RDSR;   			//Write to commad to SR
	  while(!BF);				//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			//Clear the BF flag
	  
	  SSPBUF = 0xFF;   			//Dummy write to SSPBUF to read SR of SRAM
	  while(!BF);				//Check for the BF flag to ensure trasmission
	  temp = SSPBUF;			//Copy content to data memory
	  
	  CE =1;					//Disable the SRAM 
      return (temp);			//pass the value read back to the called function.
  }